PRO ffns,day,type,first,last
;
; Flatfielding, normalization, and evaluation of sharpness
; and rms granular contrast of SST frames observed in Sept 2003.
;
; INPUTS: day   = date (14,16,18,19,20)
;	  type  = channel ('gc', 'c6', 'ca')
;	  first = seq.num. of first frame to be processed
;	  last  = seq.num. of last frame to be processed
;	Observed raw images in sav2 or decompressed f0 format.
;	In one directory, raw images must have different numbers.
;	Pre-processed FF & DD sav2 file (a name is asked)
;	SST logfile
;	Number of lines in the logfile
;	New logfile name
;	Selection of the normalization window (selbox.pro)
;
; OUTPUT: Flatfielded & normalized frames in sav2 format,
;	  integer*2, normalized to 10000, will be
;	  located in new directories 'type_day/' (should be
;	  prepared in advance, e.g. with make_directories.pro,
;	  filename structure: type_dayffn.xxxx, examples:
;	  gc_14ffn.0045, c6_19ffn.0663, ca_18ffn.0029
;	sav2 logfile with array float(7,last-first+1)
;	0 - seq. no. of frame
;	1 - time UT in hours (decimal)
;	2 - original observed SST sharpness
;	3 - exposure time (ms)
;	4 - mean intensity in normalization window (after ff)
;	5 - rms granular contrast in normalization window
;	6 - sharpness in normalization window
;	The same in a text file .txt
;
; EXAMPLE: ffns,19,'gc',1,790
; SUBRROUTINES: f1, restore2, spc, save2, selbox, sharpness,
;		readsstlog
; NOTE: On 'little endian' machines (PC) use restore2 with keyword
;	/swap to read original data prepared on 'big endian'
;
; 25 Sept 2003, Michal

on_error,1
sday=strtrim(fix(day),2)
typo=type

; setting paths

pa1='/scratch1/sobotka/lap2003/images_out/'
outpa1='/scratch1/sobotka/lap2003/images_ffn/'
outpa2=typo+'_'+sday+'/'
ffpa='/scratch1/sobotka/lap2003/pre_process/ffdd/'

if type eq 'ca' then reb=3 else reb=2	;down-rebinning factor
if (day eq 14) or (day eq 16) then begin
	pa2=type+'_'+sday+'/'
	format_string='(I4.4)'
endif else begin
	if (type eq 'ca') or (type eq 'cah') then begin
		type='cah'
		cam='42'
	endif else begin
		cam='16'
	endelse
	pa2='images'+cam+'-'+type+'-'+sday+'/'
	format_string='(I6.6)'
endelse

; initializing

ffddname=''
logname=''
newlogname=''
read,' Name of the flat&dark file (e.g. ff1dd18gc): ',ffddname
read,' Number of all lines in SST logfile: ',nlog
read,' Name of the new logfile (e.g. gc_18lg1): ',newlogname
ffddname=ffpa+ffddname+'.sav2'
logname=findfile(pa1+pa2+'*lg*')	;auto search
if (size(logname))(1) ne 1 then message,' !!! logfile not found !!!'
print,' SST logfile: ',logname
str='cp '+logname+' '+outpa1+outpa2
spawn,str		;copy original logfile to new dir.

nframes=last-first+1
readsstlog,logname,nlog,logarr	;read SST logfile
w1=where(logarr(0,*) eq first)
w2=where(logarr(0,*) eq last)
logarr=logarr(*,w1(0):w2(0))	;log limited to processed frames
newlog=fltarr(3,nframes)	;prepared extension for more data

print,' Reading: ',ffddname
restore2,ffddname,ff,dd		;read flat & dark (float)
fd=ff-dd
fd=fd > 100.	;to avoid division by zero - value '100' was
		; determined previously from histograms

; selection of the normalization window

num=(first+last)/2	;a frame in the middle of series
fil='*.'+string(num,format=format_string)
name=findfile(pa1+pa2+fil)
if (size(name))(1) ne 1 then message,' !!! file not found !!!'
print,' Reading: ',name
if (day eq 14) or (day eq 16) then  restore2,name,im  $
  else  im=f1(name,he)
si=size(im)

select:
selbox,congrid(im,si(1)/reb,si(2)/reb),x1,x2,y1,y2
x1=x1*reb
x2=x2*reb	;coordinates of the selection window
y1=y1*reb
y2=y2*reb
window,0,xs=x2-x1+1,ys=y2-y1+1
tvscl,im(x1:x2,y1:y2)
print,' Normalization window OK ? (y/n): '
if get_kbrd(1) eq 'n' then goto,select
wdelete,0,6

; LOOP over images -------------------------------

for i=0,nframes-1 do begin

; reading image

fil='*.'+string(i+first,format=format_string)
name=findfile(pa1+pa2+fil)
if (size(name))(1) ne 1 then message,' !!! file not found !!!'
print,' Reading: ',name
if (day eq 14) or (day eq 16) then  restore2,name,im  $
  else  im=f1(name,he)

; flatfielding

im=(im-dd)/fd
im=im>0			;float

; normalization

box=im(x1:x2,y1:y2)	;normalization window
avg=total(box)/n_elements(box)
im=im/avg

; rms contrast and sharpness

box=im(x1:x2,y1:y2)	;now normalized
con=stdev(box)		;rms contrast
sha=sharpness(box,4)	;sharpness

;filling logfile

newlog(0,i)=avg
newlog(1,i)=con
newlog(2,i)=sha

; converting image to integer*2, normalized to 10000

im=fix(im*10000)

;saving flatfielded and normalized image

outfil=typo+'_'+sday+'ffn.'+string(i+first,format='(I4.4)')
outnam=outpa1+outpa2+outfil
print,''
print,' Saving: ',outnam
save2,outnam,im

endfor

; creating final logfile

log=[logarr,newlog]
print,' Saving: ',outpa1+outpa2+newlogname+'.sav2'
save2,outpa1+outpa2+newlogname+'.sav2',log
openw,2,outpa1+outpa2+newlogname+'.txt'
printf,2,' Number of frames: ',nframes
printf,2,' Seq.no., UT, SST sharp., exp.(ms), norm.mean, rms contr., sharpness'
printf,2,' -------------------------------------------------------------------'
for i=0,nframes-1 do printf,2,log(*,i),  $
  format='(1x,I4,2x,F9.5,2x,I6,2x,I4,3(2x,F9.5))'
close,2

END


